/*!
 * v1.0.0-alpha.1
 * @hawapi/js-sdk (Aug 11 2023)
 * 
 * Website: https://hawapi.theproject.id
 * Docs: https://hawapi.theproject.id/docs
 * Github: https://github.com/HawAPI/js-sdk
 * 
 * (c) Lucas Josino
 * Released under the MIT License.
 */

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

/**
 * Default API url
 */
var API_URL = 'https://hawapi.theproject.id/api';
/**
 * Default API version
 */
var API_VERSION = 'v1';
/**
 * Default API language
 */
var API_DEFAULT_LANG = 'en-US';
/**
 * The API page index header
 */
var API_HEADER_PAGE_INDEX = 'X-Pagination-Page-Index';
/**
 * The API page size header
 */
var API_HEADER_PAGE_SIZE = 'X-Pagination-Page-Size';
/**
 * The API page total header
 */
var API_HEADER_PAGE_TOTAL = 'X-Pagination-Page-Total';
/**
 * The API item total header
 */
var API_HEADER_ITEM_TOTAL = 'X-Pagination-Item-Total';
/**
 * The API language header
 */
var API_HEADER_CONTENT_LANGUAGE = 'Content-Language';

/**
 * The HawAPI options configuration.
 */
var HawAPIOptions = /** @class */ (function () {
    function HawAPIOptions(config) {
        if (config === void 0) { config = {}; }
        /**
         * The endpoint of the HawAPI instance
         *
         * @default 'https://hawapi.theproject.id/api'
         */
        this.endpoint = API_URL;
        /**
         * The version of the API
         *
         * @default 'v1'
         */
        this.version = API_VERSION;
        /**
         * The language of items in request
         *
         * **Note: This value can be overwritten later**
         *
         * @default 'en-US'
         */
        this.language = API_DEFAULT_LANG;
        this.endpoint = API_URL;
        this.version = API_VERSION;
        this.language = API_DEFAULT_LANG;
        this.timeout = 10 * 1000;
        if (config.endpoint)
            this.endpoint = config.endpoint;
        if (config.version)
            this.version = config.version;
        if (config.language)
            this.language = config.language;
        if (config.timeout)
            this.timeout = config.timeout;
        if (config.token)
            this.token = config.token;
    }
    return HawAPIOptions;
}());

var Endpoints = {
    actor: 'actors',
    character: 'characters',
    episode: 'episodes',
    game: 'games',
    location: 'locations',
    season: 'seasons',
    soundtrack: 'soundtracks'
};

/**
 * The HawAPI client.
 */
var HawAPIClient = /** @class */ (function () {
    function HawAPIClient(options) {
        this.options = new HawAPIOptions(options);
        this.headers = new Headers();
        this.request = {
            method: 'GET'
        };
        // By default, all requests are made with 'ANONYMOUS' tier
        if (options === null || options === void 0 ? void 0 : options.token) {
            this.headers.set('Authorization', "Bearer ".concat(options.token));
        }
        this.headers.set('Content-Type', 'application/json');
        this.request.headers = this.headers;
    }
    /**
     * Method to test API connection
     * @returns If connected. An {@link string} with 'Ping' value
     */
    HawAPIClient.prototype.ping = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, fetch(this.options.endpoint + '/ping', this.request)
                        .then(function (response) { return response.text(); })["catch"](function (err) {
                        throw err;
                    })];
            });
        });
    };
    /**
     * Method to get all API informations
     * @returns An new {@link APIInfoModel}
     */
    HawAPIClient.prototype.getInfo = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, fetch("".concat(this.options.endpoint), this.request)
                        .then(function (response) { return response.json(); })["catch"](function (err) {
                        throw err.json();
                    })];
            });
        });
    };
    /**
     * Method to get all client options
     * @returns
     */
    HawAPIClient.prototype.getOptions = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, __assign(__assign({}, this.options), { token: '' })];
            });
        });
    };
    /**
     * Method that get all items
     * @param target The target name
     * @param filters
     * @param pageable
     * @returns An new {@link RequestResult} with all target items
     */
    HawAPIClient.prototype.getAll = function (target, filters, pageable) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this._fetch("/".concat(Endpoints[target]), filters, pageable)];
            });
        });
    };
    /**
     * Method that get a single item
     * @param target The target name
     * @param uuid The target uuid
     * @returns An new {@link RequestResult} with specific target item
     */
    HawAPIClient.prototype.getBy = function (target, uuid) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this._fetch("/".concat(Endpoints[target], "/").concat(uuid))];
            });
        });
    };
    /**
     * Method that get a single random item
     * @param target The target name
     * @param uuid The target uuid
     * @returns An new {@link RequestResult} with random target item
     */
    HawAPIClient.prototype.getRandom = function (target) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this._fetch("/".concat(Endpoints[target], "/random"))];
            });
        });
    };
    /**
     * Method that get all item translations
     * @param target The target name
     * @param uuid The target uuid
     * @returns An new {@link RequestResult} with all target item translations
     */
    HawAPIClient.prototype.getAllTranslations = function (target, uuid) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this._fetch("/".concat(Endpoints[target], "/").concat(uuid, "/translations"))];
            });
        });
    };
    /**
     * Method that get a single item
     * @param target The target name
     * @param uuid The target uuid
     * @param language The target translation language
     * @returns An new {@link RequestResult} with specific target item translation
     */
    HawAPIClient.prototype.getTranslationBy = function (target, uuid, language) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this._fetch("".concat(Endpoints[target], "/").concat(uuid, "/translations/").concat(language))];
            });
        });
    };
    /**
     * Method that get a single random item translation
     * @param target The target name
     * @param uuid The target uuid
     * @returns An new {@link RequestResult} with random target item translation
     */
    HawAPIClient.prototype.getRandomTranslation = function (target, uuid) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this._fetch("/".concat(Endpoints[target], "/").concat(uuid, "/translations/random"))];
            });
        });
    };
    /**
     * Method that get all actor socials
     * @param uuid The actor uuid
     * @returns An new {@link RequestResult} with all actor socials
     */
    HawAPIClient.prototype.getAllSocials = function (uuid) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this._fetch("/".concat(Endpoints.actor, "/").concat(uuid, "/socials"))];
            });
        });
    };
    /**
     * Method that get a single actor social
     * @param uuid The actor uuid
     * @param social The social name
     * @returns An new {@link RequestResult} with an specific actor social
     */
    HawAPIClient.prototype.getSocialBy = function (uuid, social) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this._fetch("/".concat(Endpoints.actor, "/").concat(uuid, "/socials/").concat(social))];
            });
        });
    };
    /**
     * Method that get a single random actor social
     * @param uuid The actor uuid
     * @returns An new {@link RequestResult} with random social
     */
    HawAPIClient.prototype.getRandomSocial = function (uuid) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this._fetch("/".concat(Endpoints.actor, "/").concat(uuid, "/socials/random"))];
            });
        });
    };
    /**
     * Method to fetch from HawAPI all resources
     * @param target
     * @param filters
     * @returns An new {@link RequestResult} with body and header information
     */
    HawAPIClient.prototype._fetch = function (target, filters, pageable) {
        return __awaiter(this, void 0, void 0, function () {
            var url, timeout, controller, response, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        url = this._getUrl(target, filters, pageable);
                        controller = new AbortController();
                        if (this.options.timeout) {
                            timeout = setTimeout(function () { return controller.abort(); }, this.options.timeout);
                        }
                        return [4 /*yield*/, fetch(url, __assign(__assign({}, this.request), { signal: controller.signal }))];
                    case 1:
                        response = _b.sent();
                        clearTimeout(timeout);
                        _b.label = 2;
                    case 2:
                        _b.trys.push([2, 4, , 6]);
                        // Handle any possible not 2XX code
                        if (!response.ok)
                            throw new Error('Code is not OK (200)');
                        _a = this._buildResult;
                        return [4 /*yield*/, response.json()];
                    case 3: return [2 /*return*/, _a.apply(this, [_b.sent(), response.headers,
                            response.status])];
                    case 4:
                        _b.sent();
                        return [4 /*yield*/, this._handleError(response)];
                    case 5: throw _b.sent();
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Method to handle any request error
     * @param err The request error response
     */
    HawAPIClient.prototype._handleError = function (err) {
        return __awaiter(this, void 0, void 0, function () {
            var isJson, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        isJson = err.headers.get('Content-Type') == 'application/json';
                        if (!isJson) return [3 /*break*/, 2];
                        return [4 /*yield*/, err.json()];
                    case 1: return [2 /*return*/, (_b.sent())];
                    case 2:
                        _a = Error.bind;
                        return [4 /*yield*/, err.text()];
                    case 3: throw new (_a.apply(Error, [void 0, _b.sent()]))();
                }
            });
        });
    };
    /**
     * Method to create the url using API:
     * - Url
     * - Version
     * - Target
     * - Params
     *   - Filters
     *   - Pageable
     *
     * @param target
     * @param filters
     * @param pageable
     * @returns A complete url path with all params and filters
     */
    HawAPIClient.prototype._getUrl = function (target, filters, pageable) {
        var params = '?';
        // Get all filters names and values
        if (filters) {
            for (var key in filters) {
                params += "".concat(key, "=").concat(filters[key], "&");
            }
        }
        // Define the page, sort and order
        if (pageable) {
            if (pageable.page)
                params += "page=".concat(pageable.page, "&");
            if (pageable.size)
                params += "size=".concat(pageable.size, "&");
            // 'Order' can only be applied when 'sort' is defined
            if (pageable.sort) {
                if (pageable.order) {
                    params += "sort=".concat(pageable.sort, ",").concat(pageable.order, "&");
                }
                else {
                    params += "sort=".concat(pageable.sort, "&");
                }
            }
        }
        params = params.slice(0, -1);
        return this.options.endpoint + "/".concat(this.options.version).concat(target).concat(params);
    };
    /**
     * Method to build the client response
     * @param body The body response from API request
     * @param headers The response header
     * @param status The response status code
     * @returns An new {@link RequestResult} with all required information
     * @throws An error if page index ({@link API_HEADER_PAGE_INDEX}) is less then 0
     */
    HawAPIClient.prototype._buildResult = function (body, headers, status) {
        var page = headers.get(API_HEADER_PAGE_INDEX);
        var page_size = headers.get(API_HEADER_PAGE_SIZE);
        var page_total = headers.get(API_HEADER_PAGE_TOTAL);
        var item_total = headers.get(API_HEADER_ITEM_TOTAL);
        var language = headers.get(API_HEADER_CONTENT_LANGUAGE);
        return {
            page: page ? Number(page) : null,
            page_size: page_size ? Number(page_size) : null,
            page_total: page_total ? Number(page_total) : null,
            item_size: item_total ? Number(item_total) : null,
            next_page: page ? this._handlePage(Number(page), true) : null,
            prev_page: page ? this._handlePage(Number(page), false) : null,
            language: language || null,
            status: status,
            data: body
        };
    };
    /**
     * Method to handle the next/prev page index
     * @param page The current page index
     * @param increase Determine if page index will be increase or decrease
     * @returns Updated page index
     * @throws An error if page index ({@link API_HEADER_PAGE_INDEX}) is less than 0
     */
    HawAPIClient.prototype._handlePage = function (page, increase) {
        if (page < 0)
            throw new Error('Page index cannot be nagative');
        if (increase && page >= 0)
            return page + 1;
        if (!increase && page > 0)
            return page - 1;
        return page;
    };
    return HawAPIClient;
}());

var Orders = {
    asc: 'ASC',
    desc: 'DESC'
};

/**
 * The HawAPI page options.
 */
var Pageable = /** @class */ (function () {
    function Pageable(pageable) {
        if (pageable === void 0) { pageable = {}; }
        /**
         * The request page
         *
         * @default 0
         */
        this.page = 0;
        /**
         * The size of items inside each page
         *
         * @default 10
         */
        this.size = 10;
        /**
         * The order of items inside each page
         *
         * **Note: Order will only be applied if {@link size} is defined**
         *
         * Options:
         *  - Ascending (ASC)
         *  - Descending (DESC)
         *
         * @default 'asc'
         */
        this.order = 'asc';
        this.page = 0;
        this.size = 10;
        if (pageable.page)
            this.page = pageable.page;
        if (pageable.size)
            this.size = pageable.size;
        if (pageable.sort)
            this.sort = pageable.sort;
        if (pageable.order)
            this.order = pageable.order;
    }
    return Pageable;
}());

/**
 * Creates a new HawAPI client.
 *
 * Website: https://hawapi.theproject.id \
 * Docs: https://hawapi.theproject.id/docs \
 * Github: https://github.com/HawAPI/js-sdk
 *
 * @example
 * - Javascript (UMD)
 * ```js
 *  const client = HawAPI.createClient()
 *  // Also valid:
 *  // const { createClient } = HawAPI
 *  // const client = createClient()
 *
 *  console.log('HawAPI client: ', client)
 * ```
 *
 * @example
 * - Javascript (ESM)
 * ```js
 *  import { createClient } from '<REF>'
 *  const client = createClient()
 *
 *  console.log('HawAPI client: ', client)
 * ```
 *
 * See more samples: https://github.com/HawAPI/js-sdk/tree/main/samples
 *
 * @param options Endpoint, version, token, etc.. configurations
 * @returns A new HawAPI Client
 */
var createClient = function (options) {
    return new HawAPIClient(options);
};

export { Endpoints, HawAPIClient, HawAPIOptions, Orders, Pageable, createClient };
//# sourceMappingURL=index.mjs.map
